/*	Adapted from Dave McPherson's code:
 *	http://support.microsoft.com/kb/q190351/
 */

#include "stdafx.h"
#include "compiler.h"

using namespace std;

Compiler::Compiler()
{
	commandLine = NULL;
}

Compiler::Compiler(const char *cmd)
{
	commandLine = cmd;	
}

void Compiler::SetCommandLine(const char *cmd)
{
	commandLine = cmd;
}

BOOL Compiler::Compile(const char *cmd)
{
	commandLine = cmd;
	return Compile();
}

BOOL Compiler::Compile() 
{
	HANDLE hOutputReadTmp,hOutputRead,hOutputWrite;
	HANDLE hInputWriteTmp,hInputRead,hInputWrite;
	HANDLE hErrorWrite;
	
	SECURITY_ATTRIBUTES sa;

	hChildProcess = NULL;
	hStdIn = NULL;
	bRunThread = TRUE;
	
	sa.nLength= sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = TRUE;
	
	if(!commandLine)
		return FALSE;

	if (!CreatePipe(&hOutputReadTmp,&hOutputWrite,&sa,0))
		return FALSE;

	if (!DuplicateHandle(GetCurrentProcess(),hOutputWrite,
			   GetCurrentProcess(),&hErrorWrite,0,
			   TRUE,DUPLICATE_SAME_ACCESS))
		return FALSE;

	if (!CreatePipe(&hInputRead,&hInputWriteTmp,&sa,0))
		return FALSE;

	if (!DuplicateHandle(GetCurrentProcess(),hOutputReadTmp,
			   GetCurrentProcess(),
			   &hOutputRead,
			   0,FALSE,
			   DUPLICATE_SAME_ACCESS))
		return FALSE;

	if (!DuplicateHandle(GetCurrentProcess(),hInputWriteTmp,
			   GetCurrentProcess(),
			   &hInputWrite,
			   0,FALSE,
			   DUPLICATE_SAME_ACCESS))
		return FALSE;


	if (!CloseHandle(hOutputReadTmp))
		return FALSE;
	if (!CloseHandle(hInputWriteTmp))
		return FALSE;

	if ( (hStdIn = GetStdHandle(STD_INPUT_HANDLE)) ==
									INVALID_HANDLE_VALUE )
		return FALSE;

	LaunchRedirectedChild(hOutputWrite,hInputRead,hErrorWrite);

	if (!CloseHandle(hOutputWrite)) return FALSE;
	if (!CloseHandle(hInputRead )) return FALSE;
	if (!CloseHandle(hErrorWrite)) return FALSE;

	ReadAndHandleOutput(hOutputRead);

	if (!CloseHandle(hStdIn)) return FALSE;

	bRunThread = FALSE;

	if (!CloseHandle(hOutputRead)) return FALSE;
	if (!CloseHandle(hInputWrite)) return FALSE;

	commandLine = NULL;
	return TRUE;
}

void Compiler::LaunchRedirectedChild(HANDLE hChildStdOut,
                                     HANDLE hChildStdIn,
                                     HANDLE hChildStdErr)
{
	PROCESS_INFORMATION pi;
	STARTUPINFO si;

	ZeroMemory(&si,sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	si.dwFlags = STARTF_USESTDHANDLES;
	si.hStdOutput = hChildStdOut;
	si.hStdInput  = hChildStdIn;
	si.hStdError  = hChildStdErr;

	if (!CreateProcess(NULL, (char*)commandLine,NULL,NULL,TRUE,
		 CREATE_NO_WINDOW,NULL,NULL,&si,&pi))
		return;

	WaitForSingleObject(hChildStdIn, INFINITE);

	hChildProcess = pi.hProcess;
	if (!CloseHandle(pi.hThread))
		return;
}

void Compiler::ReadAndHandleOutput(HANDLE hPipeRead)
{
	char *lpBuffer;
	lpBuffer = new char[MAX_CONSOLE_CHARS];
	DWORD nBytesRead;

	ZeroMemory(lpBuffer, MAX_CONSOLE_CHARS);

	while(TRUE) {
		if (!ReadFile(hPipeRead,lpBuffer + strlen(lpBuffer), MAX_CONSOLE_CHARS - strlen(lpBuffer),
			&nBytesRead, NULL) || !nBytesRead) {
			if (GetLastError() == ERROR_BROKEN_PIPE)
				break;
			else
				return;
		}		
	}
	
	strcpy(this->CStrOutput, lpBuffer);

	delete lpBuffer;

}

